/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/**
 * A simple example program demonstrating how to preserve the dbxml id when not
 * reusing the location.
 */

#include <iostream>

#include <Memory/Interface.hpp>

#include "btl/Legs.h"

using namespace airobots;
using namespace std;
using namespace xmltio;
using namespace memory::interface;

int main(int argc, char **argv) {

	if (argc != 2) {
		cerr << "usage: " << argv[0] << " [memory name]" << endl;
		return 1;
	}

	// create a data class
	Legs original;
	original.setGenerator("test");
	original.setTimestamp(Timestamp(123, 456));
	original.setAngle(21312.78);
	original.setDistance(-1.023);
	original.setAngleVariance(0.45);
	original.setDistanceVariance(1.34);
	original.setNumInvolved(2134);
	original.setInvolved1(4543);
	original.setInvolved2(1223);
	original.setSelected(true);

	Location originalLoc(Legs::getTemplate(), Legs::getBaseTag());
	originalLoc = original;

	// display its contents
	cout << "inserting:" << endl << originalLoc.getDocumentText(true);

	// add it to the memory
	stringstream memoryName;
	memoryName << "xcf:" << argv[1];
	MemoryInterface::pointer memory = MemoryInterface::getInstance(memoryName.str());
	memory->insert(originalLoc.getDocumentText());

	// now query it back
	ResultsPtr queryResults = memory->query("/");
	if (!queryResults->hasNext()) {
		cerr << "something really weird happened with the memory" << endl;
		return 1;
	}
	string resultString;
	queryResults->next(resultString);
	Location resultLocation(resultString, Legs::getBaseTag());
	Legs parsedLegs = extract<Legs>(resultLocation);

	cout << "queried:" << endl << resultLocation.getDocumentText(true);

	// now change the parsed results and preserve the id for a replace
	parsedLegs.setDistance(23456.435);
	Location replaceLocation(Legs::getTemplate(), Legs::getBaseTag());
	replaceLocation = parsedLegs;
	replaceLocation.add(resultLocation[XPath("@dbxml:id")]);

	cout << "will be replaced with:" << endl << replaceLocation.getDocumentText(true);

	// replace the memory contents
	memory->replace(replaceLocation.getDocumentText());

	sleep(3);

}

