/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * MapArea.cpp
 *
 *  Created on: Oct 27, 2010
 *      Author: ltwardon
 */

#include "MapArea.h"
#include <sstream>

using namespace std;
using namespace airobots;

namespace airobots {

const std::string MapArea::mapAreaTypeValue[] = { "NOGO", "ROOM", "INTEREST",
		"PERSON", "DISTURB", "OBJECTS", "NO_TYP" };

MapArea::MapArea() :
		arenaName("") {

}

MapArea::MapArea(MapAreaType type) :
		arenaName(""), type(type) {

}

MapArea::MapArea(MapAreaType type, std::string arenaName,
		list<MapRegion> regions) :
		arenaName(arenaName), type(type), regions(regions) {

}

void MapArea::addRegion(MapRegion region) {

	string label = createNewLabel(region);

	region.setLabel(label);
	regions.push_back(region);

}

MapArea::MapAreaType MapArea::getType() const {
	return type;
}

void MapArea::setType(MapAreaType type) {
	this->type = type;
}

std::string MapArea::getArenaName() const {
	return arenaName;
}

void MapArea::setArenaName(std::string arenaName) {
	this->arenaName = arenaName;
}

list<MapRegion> MapArea::getRegions() const {
	return regions;
}

void MapArea::setRegions(list<MapRegion> regions) {
	this->regions = regions;
}

void MapArea::removeRegion(std::string regionLabel) {

	list<MapRegion>::iterator i;
	for (i = regions.begin(); i != regions.end();) {
		if (i->getLabel() == regionLabel) {
			i = regions.erase(i);
		} else {
			++i;
		}
	}

}

MapRegion MapArea::getRegionByLabel(std::string regionLabel) const {

	list<MapRegion>::const_iterator i;
	for (i = regions.begin(); i != regions.end(); ++i) {
		if (i->getLabel() == regionLabel)
			return *i;
	}
	MapRegion emptyRegion;
	return emptyRegion;

}

list<MapRegion> MapArea::getRegionByGroup(std::string group) const {

	list<MapRegion> r;

	list<MapRegion>::const_iterator i;
	for (i = regions.begin(); i != regions.end(); ++i) {
		if (i->getGroup() == group)
			r.push_back(*i);
	}

	return r;

}

bool MapArea::checkRegionLabel(std::string region) {

	list<MapRegion>::iterator i;
	for (i = regions.begin(); i != regions.end(); ++i) {
		if (i->getLabel() == region)
			return false;
	}
	return true;

}

std::string MapArea::createNewLabel(MapRegion region) {

	int i = 0;
	string label = region.getLabel();

	while (!checkRegionLabel(label)) {
		i++;
		stringstream stream(label);
		stream << i;
		label = stream.str();
	}

	return label;

}

}
