/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */
/*
 * GraspReturnType.cpp
 *
 *  Created on: Jun 3, 2012
 *      Author: mschoepf
 */

#include "GraspReturnType.h"

using namespace xmltio;
using namespace std;

namespace airobots {
const float TOLERANCE = 0.001;
const LengthUnit iLU = MILLIMETER;

GraspReturnType::GraspReturnType() : graspResult(NO_RESULT), x(0.0), y(0.0), z(0.0),rating(0.0){
}

GraspReturnType::GraspReturnType(const GraspReturnType::GraspResult &gr) : graspResult (gr), x(0.0), y(0.0), z(0.0),rating(0.0) {
}

GraspReturnType::GraspReturnType(const GraspReturnType::GraspResult &gr,const float &x,const float &y,const float &z, LengthUnit unit) : graspResult (gr), rating (0.0) {
	setX (x,unit);
	setY (y,unit);
	setZ (z,unit);
}
GraspReturnType::GraspReturnType(const GraspReturnType::GraspResult &gr,const float &rating) : graspResult(gr), x(0.0), y(0.0), z(0.0), rating(rating)  {}

std::string
GraspReturnType::getBaseTag() {
	return "GRASPRETURNTYPE";
}

std::string
GraspReturnType::getTemplate() {
	return string("<") + getBaseTag()
			+ " x=\"\" y=\"\" z=\"\" rating=\"\" graspResult=\"\"><GENERATOR/>"
			+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}


GraspReturnType::GraspResult
GraspReturnType::getGraspResult() const {
	return graspResult;
}

float
GraspReturnType::getX(LengthUnit unit) const {
	return UnitConverter::convert(x, iLU, unit);
}

float
GraspReturnType::getY(LengthUnit unit) const {
	return UnitConverter::convert(y, iLU, unit);
}

float
GraspReturnType::getZ(LengthUnit unit) const {
	return UnitConverter::convert(x, iLU, unit);
}

float
GraspReturnType::getRating() const {
	return rating;
}


void
GraspReturnType::setX(const float &x, LengthUnit unit) {
	this->x = UnitConverter::convert(x, unit, iLU);
}

void
GraspReturnType::setY(const float &y, LengthUnit unit) {
	this->y = UnitConverter::convert(y, unit, iLU);
}

void
GraspReturnType::setZ(const float &z, LengthUnit unit) {
	this->z = UnitConverter::convert(z, unit, iLU);
}

void
GraspReturnType::setRating (const float &rating) {
	this->rating = rating;
}

void
GraspReturnType::setGraspResult(const GraspReturnType::GraspResult &graspResult) {
	this->graspResult = graspResult;
}

void
GraspReturnType::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);
	loc[XPath("@x")] = x;
	loc[XPath("@y")] = y;
	loc[XPath("@z")] = z;
	loc[XPath("@rating")] = rating;

	switch (graspResult)
	{
	case SUCCESS:
		loc[XPath("@graspResult")] = "SUCCESS";
		break;
	case POSITION_UNREACHABLE:
		loc[XPath("@graspResult")] = "POSITION_UNREACHABLE";
		break;
	case ROBOT_CRASHED:
		loc[XPath("@graspResult")] = "ROBOT_CRASHED";
		break;
	case FAIL:
		loc[XPath("@graspResult")] = "FAIL";
		break;
	case COLLISION_HANDLED:
		loc[XPath("@graspResult")] = "COLLISION_HANDLED";
		break;
	case NO_RESULT:
		loc[XPath("@graspResult")] = "NO_RESULT";
		break;
	default:
		loc[XPath("@graspResult")] = "INTERNAL ERROR IN BTL";
		break;
	}
}

void
GraspReturnType::fromLocation(const xmltio::Location &loc_, GraspReturnType &object) {

	Type::fromLocation(loc_, object);
	object.x = extract<float>(loc_[XPath("@x")]);
	object.y = extract<float>(loc_[XPath("@y")]);
	object.z = extract<float>(loc_[XPath("@z")]);
	object.rating = extract<float>(loc_[XPath("@rating")]);

	string graspResultStr = extract<string>(loc_[XPath("@graspResult")]);
	if (graspResultStr == "SUCCESS")
	{
		object.setGraspResult(SUCCESS);
		return;
	}
	if (graspResultStr == "POSITION_UNREACHABLE")
	{
		object.setGraspResult(POSITION_UNREACHABLE);
		return;
	}
	if (graspResultStr == "ROBOT_CRASHED")
	{
		object.setGraspResult(ROBOT_CRASHED);
		return;
	}
	if (graspResultStr == "FAIL")
	{
		object.setGraspResult(FAIL);
		return;
	}
	if (graspResultStr == "COLLISION_HANDLED")
		{
			object.setGraspResult(COLLISION_HANDLED);
			return;
		}
	if (graspResultStr == "NO_RESULT")
	{
		object.setGraspResult(NO_RESULT);
		return;
	}
	//If we got here, something went wrong...
	std::string s ("Illegal value for graspResult: ");
	s.append (graspResultStr);
	throw ParsingException(s);
}

void
GraspReturnType::sanitizeLocation(xmltio::Location & loc) const {
	xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}
}

bool
GraspReturnType::operator==(const GraspReturnType &other) const {

	if (!Type::operator ==(other))
		return false;
	if (abs(other.x - x) > TOLERANCE)
		return false;
	if (abs(other.y - y) > TOLERANCE)
		return false;
	if (abs(other.z - z) > TOLERANCE)
		return false;
	if (abs(other.rating -rating) > TOLERANCE)
		return false;
	if (other.graspResult != graspResult)
		return false;
	return true;
}

ostream& operator<<(ostream& o, const GraspReturnType& r) {

	return o << "GraspReturnType [gen=" << r.getGenerator() << ", timestamp="
			<< r.getTimestamp() << ", coordinates=" << r.getX(iLU) << ";"
			<< r.getY(iLU) << ";" << r.getZ(iLU) << ", rating=" << r.getRating() << ", graspResult=" << r.getGraspResult() <<  "]";

}

GraspReturnType::~GraspReturnType() {
}
}

