/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * GraspReturnType.h
 *
 *  Created on: Jun 3, 2012
 *      Author: mschoepf
 */

#ifndef GRASPRETURNTYPE_H_
#define GRASPRETURNTYPE_H_

#include <xmltio/xmltio.hpp>
#include <cmath>
#include <ostream>
#include <string>
#include "../../Type.h"
#include "../../units/UnitConverter.h"

namespace airobots
{

class GraspReturnType : public Type {

public:
	enum GraspResult {
		SUCCESS, POSITION_UNREACHABLE, ROBOT_CRASHED, FAIL, COLLISION_HANDLED, NO_RESULT
	};
	GraspReturnType();
	GraspReturnType(const GraspReturnType::GraspResult &gr);
	GraspReturnType(const GraspReturnType::GraspResult &gr,const float &x,const float &y,const float &z, LengthUnit unit);
	GraspReturnType(const GraspReturnType::GraspResult &gr,const float &rating);
	virtual ~GraspReturnType();
	static std::string getBaseTag();

	static std::string getTemplate();
	float getX(LengthUnit unit = MILLIMETER) const;
	float getY(LengthUnit unit = MILLIMETER) const;
	float getZ(LengthUnit unit = MILLIMETER) const;
	float getRating() const;
	GraspReturnType::GraspResult getGraspResult() const;
	void setX(const float &x, LengthUnit unit);
	void setY(const float &y, LengthUnit unit);
	void setZ(const float &z, LengthUnit unit);
	void setRating(const float &rating);
	void setGraspResult (const GraspReturnType::GraspResult &graspResult);


	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, GraspReturnType &object);
	void sanitizeLocation(xmltio::Location &loc) const;
	bool operator==(const GraspReturnType &other) const;
private:
	GraspReturnType::GraspResult graspResult;
	float x;
	float y;
	float z;
	float rating;
};

std::ostream& operator<<(std::ostream &o, const GraspReturnType &r);

}
#endif /* GRASPRETURNTYPE_H_ */
