/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * CameraInfo.cpp
 *
 *  Created on: Sep 13, 2010
 *      Author: ltwardon
 */

#define _USE_MATH_DEFINES
#include <math.h>

#include "CameraInfo.h"

using namespace airobots;

const map<CameraInfo::CameraModel, CameraInfo> CameraInfo::info = CameraInfo::createInfo();

map<CameraInfo::CameraModel, CameraInfo> CameraInfo::createInfo() {

	map<CameraInfo::CameraModel,CameraInfo> info;
	info[CameraInfo::SONY_EVI_D31] = CameraInfo(
			12, // max zoom factor
			4.3, 48.8, // angle of view
			752, 585, // resolution
			-100.0, 100.0, // angle pan
			-25.0, 25.0, // angle tilt
			DEGREE);
	info[CameraInfo::LOGITECH_PRO_9000] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			800, 600, // resolution
			0.0, 0.0, // angle pan
			0.0, 0.0, // angle tilt
			DEGREE);
	info[CameraInfo::POINTGRAY_GRAS_20S4C_C_0_3M] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			640, 480, // resolution
			0.0, 0.0, // angle pan
			0.0, 0.0, // angle tilt
			DEGREE);
	info[CameraInfo::POINTGRAY_GRAS_20S4C_C_1_4M] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			1384, 1036, // resolution
			0.0, 0.0, // angle pan
			0.0, 0.0, // angle tilt
			DEGREE);
	info[CameraInfo::POINTGRAY_GRAS_20S4C_C_2_0M] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			640, 480, // resolution
			1624.0, 1224.0, // angle pan
			0.0, 0.0, // angle tilt
			DEGREE);
	return info;

}

CameraInfo CameraInfo::getInfo(CameraInfo::CameraModel model){
	return info.at(model);
}

double CameraInfo::calculateAngleOfView(CameraModel camera, double zoom) {

	if (info.at(camera).getMaxAngleOfViewH(iAU) == info.at(camera)
			.getMinAngleOfViewH(iAU)) {
		return info.at(camera).getMaxAngleOfViewH(iAU);
	}

	double angle = (1 - zoom)
			* (info.at(camera).getMaxAngleOfViewH(iAU) - info.at(camera)
					.getMinAngleOfViewH(iAU))
			+ info.at(camera).getMinAngleOfViewH(iAU);
	return angle;
}

// double CameraInfo::degreeToRadiant(double angle) {
// 	return angle * M_PI / 180.0;
// }
// 
// double CameraInfo::radiantToDegree(double angle) {
// 	return angle * 180.0 / M_PI;
// }

CameraInfo::CameraInfo(){

}

CameraInfo::CameraInfo(int maxZoomFactor, double minAngleOfViewH,
			double maxAngleOfViewH, int resolutionX, int resolutionY,
			double minAnglePan, double maxAnglePan, double minAngleTilt,
			double maxAngleTilt, AngleUnit aU) {
	this->maxZoomFactor = maxZoomFactor;
	this->minAngleOfViewH = UnitConverter::convert(minAngleOfViewH,aU,iAU);
	this->maxAngleOfViewH = UnitConverter::convert(maxAngleOfViewH,aU,iAU);
	this->resolutionX = resolutionX;
	this->resolutionY = resolutionY;
	this->minAnglePan = UnitConverter::convert(minAnglePan,aU,iAU);
	this->maxAnglePan = UnitConverter::convert(maxAnglePan,aU,iAU);
	this->minAngleTilt = UnitConverter::convert(minAngleTilt,aU,iAU);
	this->maxAngleTilt = UnitConverter::convert(maxAngleTilt,aU,iAU);
}
	/*
	 * @param aU
	 * 		desired unit for output
	 */
double CameraInfo::getMaxAngleOfViewH(AngleUnit aU) const
{
    return UnitConverter::convert(maxAngleOfViewH,iAU, aU);
}

	/*
	 * @param aU
	 * 		desired unit for output
	 */

double CameraInfo::getMaxAnglePan(AngleUnit aU) const
{
    return UnitConverter::convert(maxAnglePan,iAU, aU);
}

/*
	 * @param aU
	 * 		desired unit for output
	 */

double CameraInfo::getMaxAngleTilt(AngleUnit aU) const
{
    return UnitConverter::convert(maxAngleTilt,iAU, aU);
}

int CameraInfo::getMaxZoomFactor() const
{
    return maxZoomFactor;
}

	/*
	 * @param aU
	 * 		desired unit for output
	 */

double CameraInfo::getMinAngleOfViewH(AngleUnit aU) const
{
    return UnitConverter::convert(minAngleOfViewH,iAU, aU);

}

	/*
	 * @param aU
	 * 		desired unit for output
	 */
double CameraInfo::getMinAnglePan(AngleUnit aU) const
{
    return UnitConverter::convert(minAnglePan,iAU, aU);
}

	/*
	 * @param aU
	 * 		desired unit for output
	 */
double CameraInfo::getMinAngleTilt(AngleUnit aU) const
{
    return UnitConverter::convert(minAngleTilt,iAU, aU);
    //return degreeToRadiant(minAngleTilt);
}

int CameraInfo::getResolutionX() const
{
    return resolutionX;
}

int CameraInfo::getResolutionY() const
{
    return resolutionY;
}
