/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * LaserInfo.h
 *
 *  Created on: Sep 17, 2010
 *      Author: ltwardon
 */

#ifndef LASERINFO_H_
#define LASERINFO_H_

#include <map>
#include <exception>
#include <string>
#include "../../units/UnitConverter.h"

using namespace std;

namespace airobots{

class LaserInfo {

public:

	enum ScannerModel {
		SICK_LMS200,
		HOKUYO_UBG_04LX_F01
	};

	class OutOfRangeException : public exception {
	private:
		std::string message;
	public:
		OutOfRangeException(string message = "requested angle is out of laser range!") {
			this->message = message;
		}

		~OutOfRangeException() throw() {}

		virtual const char* what() const throw() {
			return message.c_str();
		}
	};

	static LaserInfo getInfo(ScannerModel model);
	static int calcScanIndex(ScannerModel scannerModel, double angle) throw(OutOfRangeException);

	LaserInfo();
	LaserInfo(double range, int numScans, double maxDistance,AngleUnit aU,LengthUnit lU);
	double getRangeAngle(AngleUnit angleUnit);
	double getMaxDistance(LengthUnit lengthUnit);
	int getNumScans();

private:

    static const map<ScannerModel, LaserInfo> info;
	static map<ScannerModel, LaserInfo> createInfo();
	static const LengthUnit iLU=METER;
	static const AngleUnit iAU=DEGREE;

	double range;
	double maxDistance;
	int numScans;

};

}

#endif /* LASERINFO_H_ */
