/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * DynamicGridMap.cpp
 *
 *  Created on: Jul 12, 2010
 *      Author: lziegler, jwienke
 */

#include "DynamicGridMap.h"

#include <byteswap.h>

using namespace airobots;
using namespace std;
using namespace xmltio;

std::string DynamicGridMap::getBaseTag() {
	return "DYNAMICGRIDMAP";
}

std::string DynamicGridMap::getTemplate() {
	return "<" + getBaseTag() + " uri=\"\"><GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<MAPPROPERTIES width=\"\" height=\"\" xorigin=\"\" yorigin=\"\" resolution=\"\"/>"
			+ "</" + getBaseTag() + ">";
}

DynamicGridMap::DynamicGridMap() :
	Type("unknown") {

}

DynamicGridMap::DynamicGridMap(string generator) :
	Type(generator) {

}

DynamicGridMap::DynamicGridMap(const DynamicGridMap &map) :
	Type(map), uri(map.uri), sizeX(map.sizeX), sizeY(map.sizeY), originX(
			map.originX), originY(map.originY), resolution(map.resolution),
			gridMap(map.gridMap) {
}

DynamicGridMap& DynamicGridMap::operator=(DynamicGridMap const& map) {
	if (&map != this) {
		uri = map.uri;
		sizeX = map.sizeX;
		sizeY = map.sizeY;
		originX = map.originX;
		originY = map.originY;
		resolution = map.resolution;
		this->gridMap = map.gridMap;
	}
	return *this;
}

DynamicGridMap::~DynamicGridMap() {

}

std::string DynamicGridMap::getUri() const {
	return uri;
}

void DynamicGridMap::setUri(std::string uri) {
	this->uri = uri;
}

unsigned int DynamicGridMap::getSizeX() const {
	return sizeX;
}

void DynamicGridMap::setSizeX(unsigned int sizeX) {
	this->sizeX = sizeX;
}

unsigned int DynamicGridMap::getSizeY() const {
	return sizeY;
}

void DynamicGridMap::setSizeY(unsigned int sizeY) {
	this->sizeY = sizeY;
}

unsigned int DynamicGridMap::getOriginX() const {
	return originX;
}

void DynamicGridMap::setOriginX(unsigned int originX) {
	this->originX = originX;
}

unsigned int DynamicGridMap::getOriginY() const {
	return originY;
}

void DynamicGridMap::setOriginY(unsigned int originY) {
	this->originY = originY;
}

float DynamicGridMap::getResolution() const {
	return resolution;
}
void DynamicGridMap::setResolution(float resolution) {
	this->resolution = resolution;
}

const GridMap& DynamicGridMap::getGridMap() const {
	return gridMap;
}
void DynamicGridMap::setGridMap(GridMap &gridMap) {

	this->gridMap = gridMap;
}
void DynamicGridMap::setGridMap(const vector<unsigned char> &charArray,
		unsigned int width, unsigned int heigth) {

	gridMap = GridMap(width, heigth);

	for (uint i = 0; i < width; i++) {
		for (uint j = 0; j < heigth; j++) {
			float value;
			size_t size = sizeof value;
			unsigned char bytes[size];
			for (size_t k = 0; k < size; k++) {

				// big endian to little endian
				int kInv = (size - 1 -k);

				// get bytes
				bytes[k] = charArray[(i * heigth + j) * size + kInv];
			}
			value = *((float *) bytes);
			gridMap(i, j) = value;
		}
	}
	this->sizeX = width;
	this->sizeY = heigth;
}
void DynamicGridMap::setGridMap(const vector<float> &floatArray,
		unsigned int width, unsigned int heigth) {

	gridMap = GridMap(width, heigth);

	for (uint i = 0; i < width; i++) {
		for (uint j = 0; j < heigth; j++) {
			float value = floatArray[i * heigth + j];

			gridMap(i, j) = value;
		}
	}
	this->sizeX = width;
	this->sizeY = heigth;
}

void DynamicGridMap::sanitizeLocation(xmltio::Location &loc) const {

}

void DynamicGridMap::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("MAPPROPERTIES/@width")] = getSizeX();
	loc[XPath("MAPPROPERTIES/@height")] = getSizeY();
	loc[XPath("MAPPROPERTIES/@xorigin")] = getOriginX();
	loc[XPath("MAPPROPERTIES/@yorigin")] = getOriginY();
	loc[XPath("MAPPROPERTIES/@resolution")] = getResolution();

	loc["uri"] = getUri();

}

void DynamicGridMap::fromLocation(const xmltio::Location &loc_,
		DynamicGridMap &map) {

	Type::fromLocation(loc_, map);

	map.setUri(extract<string> (loc_["uri"]));

	map.setSizeX(extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@width")]));
	map.setSizeY(extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@height")]));

	map.setOriginX(
			extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@xorigin")]));
	map.setOriginY(
			extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@yorigin")]));

	map.setResolution(extract<float> (loc_[XPath("MAPPROPERTIES/@resolution")]));

}

