/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include <stdio.h>
#include <sstream>
#include <stdlib.h>

#include "SlamUpdateData.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

std::string SlamUpdateData::getBaseTag() {
	return "SLAMUPDATE";
}

std::string SlamUpdateData::getTemplate() {
	return std::string("<" + getBaseTag() + ">") + "  <GENERATOR />"
			+ Timestamp::getTemplate() + "  <ADDITIONS />"
			+ "  <REMOVALS />" + "</" + getBaseTag() + ">";
}

std::string SlamUpdateData::getGridTemplate() {
	return "<GRID metric_x=\"0\" metric_y=\"0\" />";
}

SlamUpdateData::SlamUpdateData() :
	Type("unknown") {

}

SlamUpdateData::SlamUpdateData(std::string generator, bool init) :
	Type(generator), init(init) {
}

void SlamUpdateData::clear() {
	additions.clear();
	removals.clear();
}

bool SlamUpdateData::isInit() const {
	return init;
}

void SlamUpdateData::setInit(bool init) {
	this->init = init;
}

const SlamUpdateData::list_t & SlamUpdateData::getAdditions() const {
	return additions;
}

const SlamUpdateData::list_t & SlamUpdateData::getRemovals() const {
	return removals;
}

void SlamUpdateData::addAddition(metricCoord_t addition) {
	additions.push_back(addition);
}
void SlamUpdateData::addRemoval(metricCoord_t removal) {
	removals.push_back(removal);
}

void SlamUpdateData::addAdditions(list_t::const_iterator begin,
		list_t::const_iterator end) {
	additions.insert(additions.end(), begin, end);
}

void SlamUpdateData::addRemovals(list_t::const_iterator begin,
		list_t::const_iterator end) {
	removals.insert(removals.end(), begin, end);
}

void SlamUpdateData::sanitizeLocation(xmltio::Location &loc) const {
	Location addLoc = loc[XPath("ADDITIONS")];
	XPath xp("GRID");
	while (XPathIterator i = xp.evaluate(addLoc)) {
		Location l(*i);
		addLoc.remove(l);
	}

	Location remLoc = loc[XPath("REMOVALS")];
	while (XPathIterator i = xp.evaluate(remLoc)) {
		Location l(*i);
		remLoc.remove(l);
	}
}

void SlamUpdateData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	// type
	if (isInit()) {
		loc["type"] = "init";
	} else {
		loc["type"] = "update";
	}

	// add additions
	Location addLoc(loc, "ADDITIONS");
	for (SlamUpdateData::list_t::const_iterator it =
			getAdditions().begin(); it != getAdditions().end(); it++) {
		Location newGrid(SlamUpdateData::getGridTemplate(), "/GRID");
		newGrid["metric_x"] = it->first;
		newGrid["metric_y"] = it->second;
		addLoc.add(newGrid);
	}

	// add removals
	Location remLoc(loc, "REMOVALS");
	for (SlamUpdateData::list_t::const_iterator it = getRemovals().begin(); it
			!= getRemovals().end(); it++) {
		Location newGrid(SlamUpdateData::getGridTemplate(), "/GRID");
		newGrid["metric_x"] = it->first;
		newGrid["metric_y"] = it->second;
		remLoc.add(newGrid);
	}

}

void SlamUpdateData::fromLocation(const xmltio::Location &loc_, SlamUpdateData &sud) {

	Type::fromLocation(loc_, sud);

	string type = extract<string> (loc_["type"]);
	bool init = false;
	if (type == "init") {
		init = true;
	}
	sud.setInit(init);

	for (XPathIterator addIter = XPath("ADDITIONS/GRID").evaluate(loc_); addIter; ++addIter) {
		std::pair<float, float> coordPair;
		coordPair.first = extract<float> ((*addIter)["metric_x"]);
		coordPair.second = extract<float> ((*addIter)["metric_y"]);
		sud.addAddition(coordPair);
	}

	for (XPathIterator remIter = XPath("REMOVALS/GRID").evaluate(loc_); remIter; ++remIter) {
		std::pair<float, float> coordPair;
		coordPair.first = extract<float> ((*remIter)["metric_x"]);
		coordPair.second = extract<float> ((*remIter)["metric_y"]);
		sud.addRemoval(coordPair);
	}

}
