/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * plannerstate.h
 *
 *  Created on: Jun 25, 2009
 *      Author: jwienke
 */

#ifndef GLOBALPLANNERSTATECHANGE_H_
#define GLOBALPLANNERSTATECHANGE_H_

#include "../../Type.h"
#include "../common/Timestamp.h"

namespace airobots {

/**
 * A state change of the global planner.
 */
class GlobalPlannerStateChange: public Type {
public:

	static std::string getBaseTag();

	static std::string getTemplate();

	/**
	 * Possible states of a global planner.
	 */
	enum State {

		/**
		 * The planner now begins planning a new path.
		 */
		PLANNING,

		/**
		 * Planning finished successfully and the robots is now trying to move
		 * along the planned path. In this state the planner can switch back
		 * to <code>PLANNING</code> every time it detects a new obstacle on the
		 * planned path.
		 */
		FOLLOWING_PLAN,

		/**
		 * The planner could not find a path to the desired location. This state
		 * is only left if a new goal is specified.
		 */
		PATH_BLOCKED

	};

	GlobalPlannerStateChange();
	GlobalPlannerStateChange(std::string generator, State newState);

	State getNewState() const;
	void setNewState(State newState);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, GlobalPlannerStateChange &type);

private:
	State newState;

};

}

#endif
