/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     speedrefdata.h
 * @author   Thorsten Spexard
 * @date     08.08.2005
 * @brief    Converting and extracting speedref data to and from xml-structure
 */

#ifndef SPEEDREFDATA_CPP
#define SPEEDREFDATA_CPP

#include <string>

#include <xmltio/xmltio.hpp>

#include "../../Type.h"
#include "../common/Ticked.h"
#include "../../units/UnitConverter.h"

/**
 * xml structure for speedref data
 */

namespace airobots {

/**
 * C++ structure which is converted to / extracted from xml-data
 */
class SpeedData: public Type, public Ticked {

public:

	static std::string getBaseTag();

	static std::string getTemplate();

	SpeedData();

	/**
	 * Returns the translational speed in [SpeedUnit].
	 */
	double getTranslation(SpeedUnit u) const;
	void setTranslation(double translation, SpeedUnit u);

	/**
	 * Returns the rotational speed in [AngleUnit/s].
	 */
	double getRotation(AngleUnit u) const;
	void setRotation(double rotation, AngleUnit u);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, SpeedData &type);

private:

	/**
	 * absolute translation velocity in meters/s
	 */
	double translation;

	/**
	 * absolute rotation velocity in rad/s
	 */
	double rotation;

	static SpeedUnit iSU;
	static AngleUnit iAU;
};

}

#endif
