/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "DoorList.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string DoorList::getBaseTag() {
	return "DOORLIST";
}

std::string DoorList::getTemplate() {
	return "<" + getBaseTag() + "></" + getBaseTag() + ">";
}

DoorList::DoorList() :
	Type() {
}

DoorList::DoorList(std::string generator) :
	Type(generator) {
}

void DoorList::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(Door::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void DoorList::fillInto(xmltio::Location &loc) const {

	for (vector<Door>::const_iterator it = this->begin(); it
			!= this->end(); it++) {

		Location obj(Door::getTemplate(), XPath(
				Door::getBaseTag()));

		it->fillInto(obj);
		loc.add(obj);
	}
}

void DoorList::fromLocation(const xmltio::Location &loc_,
		DoorList &object) {

	for (XPathIterator iter = XPath(Door::getBaseTag()).evaluate(
			loc_); iter; ++iter) {
		Location l(*iter);
		Door data = extract<Door> (l);
		object.push_back(data);
	}
}

bool DoorList::operator==(const DoorList &other) const {

	bool isSame = size() == other.size();
	if (isSame) {
		for (unsigned int i = 0; i < size(); i++) {
			if (!(this->at(i) == other[i])) {
				isSame = false;
			}
		}
	}
	return isSame;
}
