/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef OBJECTDATA_H_
#define OBJECTDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include <boost/smart_ptr.hpp>

#include "../../Type.h"

namespace airobots {

/**
 * Results of the object recognition. This is a very general base class.
 *
 * @author lziegler
 */
class ObjectData: public Type {

public:

	class Hypothesis: public Type {
	private:
		/**
		 * Reliability of the recognition result.
		 */
		double reliability;

		/**
		 * Name of this object.
		 */
		std::string classLabel;

	public:

		static std::string getBaseTag();

		static std::string getTemplate();

		Hypothesis();
		Hypothesis(std::string generator);
		Hypothesis(const Hypothesis &h);

		double getReliability() const;
		void setReliability(const double reliability);

		std::string getClassLabel() const;
		void setClassLabel(const std::string label);

		virtual bool operator==(const Hypothesis &other) const;

		void sanitizeLocation(xmltio::Location&) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, Hypothesis &type);

	};

	static std::string getBaseTag();

	static std::string getTemplate();

	ObjectData();
	ObjectData(std::string generator);
	virtual ~ObjectData();

	void addHypothesis(Hypothesis &h);
	void clearHypotheses();
	std::vector<Hypothesis> getHypotheses() const;

	virtual bool operator==(const ObjectData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, ObjectData &type);

private:

protected:
	std::vector<boost::shared_ptr<Hypothesis> > hypotheses;

};

}

#endif /* OBJECTDATA_H_ */
