/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef OBJECTPOSITIONDATA_H_
#define OBJECTPOSITIONDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "../../Type.h"
#include "ObjectData.h"
#include "../../units/UnitConverter.h"

namespace airobots {

/**
 * Results of the object recognition.
 *
 * @author lziegler
 */
class ObjectPositionData: public ObjectData {

public:

	class Hypothesis: public ObjectData::Hypothesis {
	private:

	public:

		static std::string getBaseTag();
		static std::string getTemplate();

		Hypothesis();
		Hypothesis(std::string generator);

		virtual bool operator==(const Hypothesis &other) const;

		void sanitizeLocation(xmltio::Location&) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, Hypothesis &type);

	};

	static std::string getBaseTag();

	static std::string getTemplate();

	std::vector<std::pair<double, double> > getPolygon(
			LengthUnit lengthUnit) const;
	void setPolygon(const std::vector<std::pair<double, double> > &polygon,
	LengthUnit lengthUnit);

	std::string getCoordinateKind() const;
	void setCoordinateKind(const std::string &kind);
	std::string getReference() const;
	void setReference(const std::string &ref);

	ObjectPositionData();
	ObjectPositionData(std::string generator);

	std::vector<Hypothesis> getPositionHypotheses() const;
	void addHypothesis(Hypothesis &h);

	virtual bool operator==(const ObjectPositionData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			ObjectPositionData &type);

private:
	std::vector<std::pair<double, double> > polygon;
	std::string reference;
	std::string coordinateKind;
	std::vector<std::pair<double, double> > scalePolygon(
			std::vector<std::pair<double, double> > poly , double factor) const;
};

}

#endif /* OBJECTPOSITIONDATA_H_ */
