/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * FaceIdentificationList.cpp
 *
 *  Created on: Nov 12, 2010
 *      Author: ltwardon
 */

#include "FaceIdentificationList.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

FaceIdentificationList::FaceIdentification::FaceIdentification() : classId(-1)
{

}

int FaceIdentificationList::FaceIdentification::getClassId() const
{
    return classId;
}

int FaceIdentificationList::FaceIdentification::getRegionHeight() const
{
    return regionHeight;
}

int FaceIdentificationList::FaceIdentification::getRegionWidth() const
{
    return regionWidth;
}

int FaceIdentificationList::FaceIdentification::getRegionX() const
{
    return regionX;
}

int FaceIdentificationList::FaceIdentification::getRegionY() const
{
    return regionY;
}

void FaceIdentificationList::FaceIdentification::setClassId(int classId)
{
    this->classId = classId;
}

void FaceIdentificationList::FaceIdentification::setRegionHeight(int regionHeight)
{
    this->regionHeight = regionHeight;
}

void FaceIdentificationList::FaceIdentification::setRegionWidth(int regionWidth)
{
    this->regionWidth = regionWidth;
}

void FaceIdentificationList::FaceIdentification::setRegionX(int regionX)
{
    this->regionX = regionX;
}

void FaceIdentificationList::FaceIdentification::setRegionY(int regionY)
{
    this->regionY = regionY;
}

bool FaceIdentificationList::FaceIdentification::isIdentified() const
{
	return classId != -1;
}

FaceIdentificationList::FaceIdentificationList()
{
	setLegacyParsing(true);
}

std::string FaceIdentificationList::getBaseTag()
{
	return "FACES";
}

std::string FaceIdentificationList::getTemplate()
{
	return "";
}

int FaceIdentificationList::size()
{
	return identifications.size();
}

list<FaceIdentificationList::FaceIdentification> FaceIdentificationList::getIdentifications()
{
	return identifications;
}

void FaceIdentificationList::sanitizeLocation(xmltio::Location &loc) const
{

}

void FaceIdentificationList::fillInto(xmltio::Location &loc) const
{

}

void FaceIdentificationList::fromLocation(const xmltio::Location &loc, FaceIdentificationList &type)
{
	Type::fromLocation(loc, type);

	for(XPathIterator it_face = XPath("FACE").evaluate(loc); it_face; ++it_face) {
			Location loc_face(*it_face);
			FaceIdentification ident;

			for(XPathIterator it_region = XPath("REGION").evaluate(loc_face); it_region; ++it_region) {
				Location loc_region(*it_region);

				for(XPathIterator it_rect = XPath("RECTANGLE").evaluate(loc_region); it_rect; ++it_rect) {
					Location loc_rect(*it_rect);
					ident.setRegionX(extract<int>(loc_rect[XPath("@x")]));
					ident.setRegionY(extract<int>(loc_rect[XPath("@y")]));
					ident.setRegionWidth(extract<int>(loc_rect[XPath("@w")]));
					ident.setRegionHeight(extract<int>(loc_rect[XPath("@h")]));
				}

			}

			for(XPathIterator it_class = XPath("CLASS").evaluate(loc_face); it_class; ++it_class) {
				Location loc_class(*it_class);
				string strID = extract<string>(loc_class[XPath("@class-id")]);
				ident.setClassId(atoi(strID.c_str()));
				for(unsigned int i = 0; i < strID.length(); i++){
					if(!isdigit(strID[i])) ident.setClassId(-1);
				}
			}

			type.addIdentification(ident);

	}

}

void FaceIdentificationList::addIdentification(FaceIdentificationList::FaceIdentification identification)
{
	identifications.push_back(identification);
}
