/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * Voice.h
 *
 *  Created on: Aug 21, 2009
 *      Author: jwienke
 */

#ifndef VOICE_H_
#define VOICE_H_

#include "Voice.h"
#include "../person/Percept.h"
#include "../../units/UnitConverter.h"

#include <string>

namespace airobots {

class Voice: public Percept {
public:

	static std::string getBaseTag();
	static std::string getTemplate();
	static std::string getPerceptName();

	Voice();
	virtual ~Voice();

	float getAngle(AngleUnit angleUnit) const;
	void setAngle(float angle,AngleUnit angleUnit);

	float getAngleVariance(AngleUnit angleUnit) const;
	void setAngleVariance(float angleVariance,AngleUnit angleUnit);

	long getRuntimeDifference(TimeUnit timeUnit) const;
	void setRuntimeDifference(long runtimeDifference,TimeUnit timeUnit);

	long getRuntimeDifferenceVariance(TimeUnit timeUnit) const;
	void setRuntimeDifferenceVariance(long runtimeDifferenceVariance,TimeUnit timeUnit);

	bool operator==(const Voice &other) const;

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Voice &type);

private:
	float angle;
	float runtimeDifference;
	float angleVariance;
	float runtimeDifferenceVariance;
};

std::ostream& operator<<(std::ostream& o, Voice const& voice);

}

#endif /* VOICE_H_ */
