/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "RobotState.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

const string RobotState::CURRENT_STATE_ELEMENT_NAME = "CURRENTSTATE";
const string RobotState::PREVIOUS_STATE_ELEMENT_NAME = "PREVIOUSSTATE";
const string RobotState::STATE_TARGET_ELEMENT_NAME = "STATETARGET";

RobotState::RobotState() {
}

RobotState::~RobotState() {
}

std::string RobotState::getBaseTag()
{
	return "ROBOTSTATE";
}



void RobotState::fillInto(xmltio::Location & loc) const
{
	Type::fillInto(loc);

	switch (getCurrentState()) {
	case DRIVE_TO:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "DRIVE_TO";
		break;
	case TALK_TO:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "TALK_TO";
		break;
	case GO_HOME:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "GO_HOME";
		break;
	case WAITING:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "WAITING";
		break;
	default:
		break;
	}

	switch (getPreviousState()) {
	case DRIVE_TO:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "DRIVE_TO";
		break;
	case TALK_TO:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "TALK_TO";
		break;
	case GO_HOME:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "GO_HOME";
		break;
	case WAITING:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "WAITING";
		break;
	default:
		break;
	}

	loc[XPath(STATE_TARGET_ELEMENT_NAME)] = getTargetName();
}



std::string RobotState::getTemplate()
{
	return "<" + getBaseTag() + ">"
			+ "<GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<" + CURRENT_STATE_ELEMENT_NAME + "/>"
			+ "<" + PREVIOUS_STATE_ELEMENT_NAME + "/>"
			+ "<" + STATE_TARGET_ELEMENT_NAME + "/>"
			+ "</" + getBaseTag() + ">";
}



void RobotState::sanitizeLocation(xmltio::Location & loc) const
{
}



void RobotState::fromLocation(const xmltio::Location & loc, RobotState & rs)
{
	Type::fromLocation(loc, rs);

	string pstateString = extract<string>(loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)]);
	string cstateString = extract<string>(loc[XPath(CURRENT_STATE_ELEMENT_NAME)]);
	string target = extract<string>(loc[XPath(STATE_TARGET_ELEMENT_NAME)]);

	RobotStates pstate;

	if (pstateString.compare("DRIVE_TO") == 0) {
		pstate = DRIVE_TO;
	} else if (pstateString.compare("TALK_TO") == 0) {
		pstate = TALK_TO;
	} else if (pstateString.compare("GO_HOME") == 0) {
		pstate = GO_HOME;
	} else if (pstateString.compare("WAITING") == 0) {
		pstate = WAITING;
	}

	RobotStates cstate;

	if (cstateString.compare("DRIVE_TO") == 0) {
		cstate = DRIVE_TO;
	} else if (cstateString.compare("TALK_TO") == 0) {
		cstate = TALK_TO;
	} else if (cstateString.compare("GO_HOME") == 0) {
		cstate = GO_HOME;
	} else if (cstateString.compare("WAITING") == 0) {
		cstate = WAITING;
	}

	rs.setPreviousState(pstate);
	rs.setCurrentState(cstate);
	rs.setTargetName(target);
}

RobotState::RobotStates RobotState::getPreviousState() const
{
	return previousState;
}



void RobotState::setPreviousState(RobotStates state)
{
	previousState = state;
}



void RobotState::setCurrentState(RobotStates state)
{
	currentState = state;
}



RobotState::RobotStates RobotState::getCurrentState() const
{
	return currentState;
}



std::string RobotState::getTargetName() const
{
	return targetName;
}



bool airobots::RobotState::operator ==(const RobotState & other) const
{
	return currentState == other.getCurrentState()
		&& previousState == other.getPreviousState()
		&& targetName.compare(other.getTargetName()) == 0;
}

void RobotState::setTargetName(std::string target)
{
	targetName = target;
}




