/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * KinectData.cpp
 *
 *  Created on: Jan 5, 2011
 *      Author: lziegler
 */

#include "KinectData.h"

using namespace airobots;
using namespace xmltio;
using namespace std;

KinectData::KinectData() {
}

KinectData::KinectData(double tilt,AngleUnit aU) {
	this->tiltAngle = UnitConverter::convert(tilt,aU,iAU);
}
KinectData::KinectData(double tilt, double accelerationX, double accelerationY,
		double accelerationZ,AngleUnit aU) :
	accelerationX(accelerationX),
			accelerationY(accelerationY), accelerationZ(accelerationZ) {
	this->tiltAngle = UnitConverter::convert(tilt,aU,iAU);
}

KinectData::~KinectData() {
}

string KinectData::getBaseTag() {
	return "KINECTDATA";
}

string KinectData::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<TILT value=\"\" /><ACCELERATION x=\"\" y=\"\" z=\"\" />" + "</"
			+ getBaseTag() + ">";
}

void KinectData::fillInto(Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("TILT/@value")] = tiltAngle;
	loc[XPath("ACCELERATION/@x")] = accelerationX;
	loc[XPath("ACCELERATION/@y")] = accelerationY;
	loc[XPath("ACCELERATION/@z")] = accelerationZ;
}

void KinectData::fromLocation(const Location &loc, KinectData &object) {

	Type::fromLocation(loc, object);

	double tilt = extract<double> (loc[XPath("TILT/@value")]);
	double ax = extract<double> (loc[XPath("ACCELERATION/@x")]);
	double ay = extract<double> (loc[XPath("ACCELERATION/@y")]);
	double az = extract<double> (loc[XPath("ACCELERATION/@z")]);

	object.setTilt(tilt, RADIAN);
	object.setAcceleration(ax, ay, az);
}

void KinectData::sanitizeLocation(Location &loc) const {
	XPath xp(getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

bool KinectData::operator==(const KinectData &other) const {
	return Type::operator ==(other) && other.accelerationX
			== this->accelerationX && other.accelerationY
			== this->accelerationY && other.accelerationZ
			== this->accelerationZ && other.tiltAngle == this->tiltAngle;
}

double KinectData::getTilt(AngleUnit unit) const {
	return UnitConverter::convert(tiltAngle, RADIAN, unit);
}
double KinectData::getAccelerationX() const {
	return accelerationX;
}
double KinectData::getAccelerationY() const {
	return accelerationY;
}
double KinectData::getAccelerationZ() const {
	return accelerationZ;
}

void KinectData::setTilt(double angle, AngleUnit unit) {
	tiltAngle = UnitConverter::convert(angle, unit, RADIAN);
}
void KinectData::setAcceleration(double x, double y, double z) {
	accelerationX = x;
	accelerationY = y;
	accelerationZ = z;
}
