/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */
/*
 * PointCloud.h
 *
 *  Created on: Jun 14, 2011
 *      Author: plueckin
 */

#ifndef POINTCLOUD_H_
#define POINTCLOUD_H_


#include <vector>
#include <iostream>
#include "../../Type.h"
#include "../../List.h"
#include "../geometry/Point3D.h"
#include "../geometry/Rotation3D.h"
#include "../../units/UnitConverter.h"

namespace airobots {

class PointCloud : public Type {

public:
	PointCloud();
	PointCloud(std::string generator);
	PointCloud(const PointCloud &pc);
	PointCloud(std::vector<Point3D> points, Point3D ori,
		Rotation3D rot, LengthUnit u);
	virtual ~PointCloud();

	static std::string getBaseTag();
	static std::string getTemplate();

	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, PointCloud &type);
	void sanitizeLocation(xmltio::Location &loc) const;

	std::vector<Point3D> getPoints() const;
	Point3D getSensorOrigin() const;
	Rotation3D getRotation() const;

	void addPoint(Point3D point);
	void setSensorOrigin(Point3D ori);
	void setRotation(Rotation3D rot);

	bool operator==(const PointCloud &other) const;

private:

	std::vector<Point3D> points;
	Point3D sensor_origin;
	Rotation3D rotation;

	LengthUnit iLU;
};

}

#endif /* POINTCLOUD_H_ */
