package de.unibi.airobots.btl.data.grasp;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.units.*;

public class PoseData extends Type {

	private double x = 0;
	private double y = 0;
	private double z = 0;
	private LengthUnit iLU= LengthUnit.MILLIMETER;

	/**
	 * Constructor.
	 */
	public PoseData() {
		super();
	}

	public static String getBaseTag() {
		return "currentarmcoord";
	}

	/**
	 * Constructor.
	 */
	public PoseData(double x, double y, double z, LengthUnit lU) {
		super();
		this.x = UnitConverter.convert(x, lU, iLU);
		this.y = UnitConverter.convert(y, lU, iLU);
		this.z = UnitConverter.convert(z, lU, iLU);

	}

	/**
	 * Returns the x-value of the endeffector position (mm)
	 * 
	 * @return x-value of the endeffector position
	 * 
	 * @parem lU
	 * 		desired length unit
	 */
	public double getX(LengthUnit lU) {
		return UnitConverter.convert(x, iLU, lU);
	}

	/**
	 * Sets the x-value of the endeffector position (mm)
	 * 
	 * @param x
	 *            x-value of the endeffector
	 * @parem lU
	 * 		input length unit
	 */
	public void setX(double x,LengthUnit lU) {
		this.x = UnitConverter.convert(x, lU, iLU);
	}

	/**
	 * Returns the y-value of the endeffector position (mm)
	 * 
	 * @return y-value of the endeffector position
	 * 
	 * @parem lU
	 * 		desired length unit
	 */
	public double getY(LengthUnit lU) {
		return UnitConverter.convert(y, iLU, lU);
	}

	/**
	 * Sets the y-value of the endeffector position (mm)
	 * 
	 * @param y
	 *            y-value of the endeffector
	 *            
	 * @parem lU
	 * 		input length unit
	 */
	public void setY(double y, LengthUnit lU) {
		this.y = UnitConverter.convert(y, lU, iLU);
	}

	/**
	 * Returns the z-value of the endeffector position (mm)
	 * 
	 * @return z-value of the endeffector position
	 * 
	 * @parem lU
	 * 		desired length unit
	 */
	public double getZ(LengthUnit lU) {
		return UnitConverter.convert(z, iLU, lU);
	}

	/**
	 * Sets the z-value of the endeffector position (mm)
	 * 
	 * @param z
	 *            z-value of the endeffector
	 * @parem lU
	 * 		input length unit
	 */
	public void setZ(double z,LengthUnit lU) {
		this.z = UnitConverter.convert(z, lU, iLU);
	}

	public static void fromElement(Element element, PoseData type) {

		Type.fromElement(element, type);

		Element poseElement = element.getFirstChildElement("currentxyz");
		type.setX(ElementParser.getDoubleAttributeValue(poseElement, "x"),LengthUnit.MILLIMETER);
		type.setY(ElementParser.getDoubleAttributeValue(poseElement, "y"),LengthUnit.MILLIMETER);
		type.setZ(ElementParser.getDoubleAttributeValue(poseElement, "z"),LengthUnit.MILLIMETER);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element poseElement = new Element("currentxyz");
		parent.appendChild(poseElement);
		poseElement.addAttribute(new Attribute("x", String.valueOf(getX(LengthUnit.MILLIMETER))));
		poseElement.addAttribute(new Attribute("y", String.valueOf(getY(LengthUnit.MILLIMETER))));
		poseElement.addAttribute(new Attribute("z", String.valueOf(getZ(LengthUnit.MILLIMETER))));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "PoseData [x = " + x + ", y = " + y + ", z = " + z + "]";
	}

}
