package de.unibi.airobots.btl.data.status;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.airobots.btl.Type;
import nu.xom.Element;

/**
 * Xml type for the state visualizer configuration.
 * 
 * @author jwienke
 */
public class StateChangeRequest extends Type {

	private static final String STATE_ELEMENT = "STATE";
	private String newStateName = "unknown";

	/**
	 * Constructor.
	 */
	public StateChangeRequest() {
	}

	/**
	 * Returns the name of the new state to change to.
	 * 
	 * @return staten name
	 */
	public String getNewStateName() {
		return newStateName;
	}

	/**
	 * Sets the name of the state to change to.
	 * 
	 * @param name
	 *            new name
	 */
	public void setNewStateName(String name) {
		this.newStateName = name;
	}

	public static String getBaseTag() {
		return "STATECHANGE";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element stateElement = new Element(STATE_ELEMENT);
		parent.appendChild(stateElement);
		stateElement.appendChild(getNewStateName());

	}

	public static void fromElement(Element element, StateChangeRequest type) {

		Type.fromElement(element, type);

		type.setNewStateName(element.getFirstChildElement(STATE_ELEMENT)
				.getValue());

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[StateChangeRequest: newStateName = " + getNewStateName() + "]";
	}

}
