/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * MapAnnotationTest.cpp
 *
 *  Created on: Nov 17, 2010
 *      Author: ltwardon
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>


#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/annotation/MapAnnotation.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

BOOST_AUTO_TEST_SUITE(MapAnnotationTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility)
{
	try
	{

		MapPoint point1(0.8, 1.8, MapPoint::WORLD, METER, "");
		MapPoint point2(1.2, 2.2, MapPoint::WORLD, METER, "");
		MapRegion region("polygon", "table", "group");
		region.addPoint(point1);
		region.addPoint(point2);
		NavigationGoalData navGoalData("test", 1.0, 2.0, 0.0, NavigationGoalData::GLOBAL);
		region.setViewPoint(navGoalData);
		list<MapRegion> regions;
		regions.push_front(region);
		MapArea area(MapArea::INTEREST, "arena", regions);

		MapAnnotation original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.addArea(area);

		Location originalLoc(MapAnnotation::getTemplate(), MapAnnotation::getBaseTag());

		originalLoc = original;

		MapAnnotation parsed = extract<MapAnnotation>(originalLoc);

		BOOST_REQUIRE_EQUAL(original.getGenerator(), parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(original.getTimestamp(), parsed.getTimestamp());
		BOOST_REQUIRE_EQUAL(area.getType(), parsed.getAreasByName("arena").front().getType());
		BOOST_REQUIRE_EQUAL(area.getArenaName(), parsed.getAreasByType(MapArea::INTEREST).front().getArenaName());
		BOOST_REQUIRE_EQUAL(region.getType(), parsed.getAreas().front().getRegionByLabel("table").getType());
		BOOST_REQUIRE_EQUAL(region.getGroup(), parsed.getAreas().front().getRegionByLabel("table").getGroup());
		BOOST_REQUIRE_CLOSE(region.getHeight(METER), parsed.getAreas().front().getRegionByLabel("table").getHeight(METER), 0.0001);
		BOOST_REQUIRE_EQUAL(point1.getKind(), parsed.getAreas().front().getRegionByLabel("table").getPoints().front().getKind());
		BOOST_REQUIRE_EQUAL(point1.getRef(), parsed.getAreas().front().getRegionByLabel("table").getPoints().front().getRef());
		BOOST_REQUIRE_CLOSE(point2.getX(METER), parsed.getAreas().front().getRegionByLabel("table").getPoints().back().getX(METER), 0.0001);
		BOOST_REQUIRE_CLOSE(point2.getY(METER), parsed.getAreas().front().getRegionByLabel("table").getPoints().back().getY(METER), 0.0001);
		BOOST_REQUIRE_CLOSE(region.getViewPoint().getX(METER), parsed.getAreas().front().getRegionByLabel("table").getViewPoint().getX(METER), 0.0001);
		BOOST_REQUIRE_CLOSE(region.getViewPoint().getY(METER), parsed.getAreas().front().getRegionByLabel("table").getViewPoint().getY(METER), 0.0001);
		BOOST_REQUIRE_CLOSE(region.getViewPoint().getTheta(RADIAN), parsed.getAreas().front().getRegionByLabel("table").getViewPoint().getTheta(RADIAN), 0.0001);

	}
	catch (std::exception e)
	{
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility)
{

	Location fileContents(readFile("Annotation.xml"), MapAnnotation::getBaseTag());
	MapAnnotation parsed = extract<MapAnnotation>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(uint64_t(12345), parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(MapArea::NOGO, parsed.getAreasByName("arena").front().getType());
	BOOST_REQUIRE_EQUAL("arena", parsed.getAreasByType(MapArea::NOGO).front().getArenaName());
	BOOST_REQUIRE_EQUAL(MapRegion::POLYGONE_TYPE, parsed.getAreas().front().getRegionByLabel("table").getType());
	BOOST_REQUIRE_EQUAL("group", parsed.getAreas().front().getRegionByLabel("table").getGroup());
	BOOST_REQUIRE_CLOSE(0.0, parsed.getAreas().front().getRegionByLabel("table").getHeight(METER), 0.0001);
	BOOST_REQUIRE_EQUAL(MapPoint::WORLD, parsed.getAreas().front().getRegionByLabel("chair").getPoints().front().getKind());
	BOOST_REQUIRE_EQUAL("", parsed.getAreas().front().getRegionByLabel("chair").getPoints().front().getRef());
	BOOST_REQUIRE_CLOSE(-3.65, parsed.getAreas().back().getRegionByLabel("start").getPoints().front().getX(METER), 0.0001);
	BOOST_REQUIRE_CLOSE(1.7, parsed.getAreas().back().getRegionByLabel("start").getPoints().back().getY(METER), 0.0001);
	BOOST_REQUIRE_CLOSE(-3.3, (double) parsed.getAreas().back().getRegionByLabel("target").getViewPoint().getX(METER), 0.0001);
	BOOST_REQUIRE_CLOSE(-4.5, (double) parsed.getAreas().back().getRegionByLabel("target").getViewPoint().getY(METER), 0.0001);
	BOOST_REQUIRE_CLOSE(1.3, (double) parsed.getAreas().back().getRegionByLabel("target").getViewPoint().getTheta(RADIAN), 0.0001);

}

BOOST_AUTO_TEST_SUITE_END()
