/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * CylinderTest.cpp
 *
 *  Created on: Mar 21, 2011
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/geometry/Cylinder.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;
using namespace Eigen;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(CylinderTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		double m[9] = {3.0, 45.0, 2.0, 0.5, 0.1, 2.9, 3.89, 0.4, 23.9};
		Matrix3d mat(m);
		Rotation3D rot(mat);
		Cylinder original;
		original.setPosition(Point3D(3.0,2.0,5.8, MILLIMETER));
		original.setOrientation(rot);
		original.setRadius(4.9, MILLIMETER);
		original.setHeight(5.9, MILLIMETER);

		Location originalLoc(Cylinder::getTemplate(), Cylinder::getBaseTag());
		originalLoc = original;

		Cylinder parsed = extract<Cylinder>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getRadius(MILLIMETER), original.getRadius(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getHeight(MILLIMETER), original.getHeight(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getPosition().getX(MILLIMETER), original.getPosition().getX(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getPosition().getY(MILLIMETER), original.getPosition().getY(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getPosition().getZ(MILLIMETER), original.getPosition().getZ(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getOrientation(), original.getOrientation());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Cylinder.xml"), Cylinder::getBaseTag());
	Cylinder parsed = extract<Cylinder>(fileContents);

	BOOST_REQUIRE_EQUAL("TestApp", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(0.0, parsed.getPosition().getX(MILLIMETER), 0.0001);
	BOOST_REQUIRE_CLOSE(1.0, parsed.getPosition().getY(MILLIMETER), 0.0001);
	BOOST_REQUIRE_CLOSE(2.0, parsed.getPosition().getZ(MILLIMETER), 0.0001);
	BOOST_REQUIRE_CLOSE(6.0, parsed.getHeight(MILLIMETER), 0.0001);
	BOOST_REQUIRE_CLOSE(7.0, parsed.getRadius(MILLIMETER), 0.0001);

	Matrix3d mat = parsed.getOrientation().getMatrix();
	BOOST_REQUIRE_CLOSE(0, mat(0,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(1, mat(0,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2, mat(0,2), TOLERANCE);
	BOOST_REQUIRE_CLOSE(3, mat(1,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(4, mat(1,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(5, mat(1,2), TOLERANCE);
	BOOST_REQUIRE_CLOSE(6, mat(2,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(7, mat(2,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(8, mat(2,2), TOLERANCE);
}

BOOST_AUTO_TEST_SUITE_END()
