/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * MovementDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/hardwareinfo/CameraTargetData.h"
#include "../../utils.h"
#include "btl/units/AngleUnit.h"
#include "btl/units/LengthUnit.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(CameraTargetDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		CameraTargetData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setAzimuth(-0.456,RADIAN);
		original.setDistance(12.45,MILLIMETER);
		original.setHeight(1.45,MILLIMETER);
		original.setZoom(0.2345);

		Location originalLoc(CameraTargetData::getTemplate(), CameraTargetData::getBaseTag());
		originalLoc = original;

		CameraTargetData parsed = extract<CameraTargetData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getAzimuth(RADIAN), original.getAzimuth(RADIAN), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getDistance(MILLIMETER), original.getDistance(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getHeight(MILLIMETER), original.getHeight(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getZoom(), original.getZoom(), TOLERANCE);

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("CameraTarget.xml"), CameraTargetData::getBaseTag());
	CameraTargetData parsed = extract<CameraTargetData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(123.0, parsed.getDistance(MILLIMETER), 0.0001);
	BOOST_REQUIRE_CLOSE(2342.0, parsed.getHeight(MILLIMETER), 0.0001);
	BOOST_REQUIRE_CLOSE(0.345, parsed.getAzimuth(RADIAN), 0.0001);
	BOOST_REQUIRE_CLOSE(0.75, parsed.getZoom(), 0.75);

}

BOOST_AUTO_TEST_SUITE_END()
