/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include "boost/random.hpp"
#include "boost/generator_iterator.hpp"

#include <iostream>

#include "btl/data/navigation/ROSBaseLocalPlannerConfiguration.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ROSBaseLocalPlannerConfigurationTest)
                
BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

        typedef boost::mt19937 RNGType;
        RNGType rng(time(0)); 
        boost::uniform_real<> one_to_fifty( 0.0, 50.0 );
        boost::variate_generator< RNGType, boost::uniform_real<> >
                    rnd(rng, one_to_fifty);
    
	try {
            
            ROSBaseLocalPlannerConfiguration original = ROSBaseLocalPlannerConfiguration();
            original.setAccLimTheta(rnd());
            original.setAccLimX(rnd());
            original.setAccLimY(rnd());
            original.setAngularSimGranularity(rnd());
            original.setDwa(rnd() > rnd());        
            original.setEscapeResetDist(rnd());
            original.setEscapeResetTheta(rnd());
            original.setEscapeVel(rnd());
            original.setGdistScale(rnd());   
            original.setHeadingLookahead(rnd());
            original.setHeadingScoring(rnd() > rnd());
            original.setHeadingScoringTimestep(rnd());
            original.setHolonomicRobot(rnd() > rnd());
            original.setMaxVelTheta(rnd());
            original.setMaxVelX(rnd());
            original.setMinInPlaceVelTheta(rnd());
            original.setMinVelTheta(rnd());
            original.setMinVelX(rnd());
            original.setOccdistScale(rnd());
            original.setOscillationResetDist(rnd());
            original.setPdistScale(rnd());
            original.setRestoreDefaults(rnd() > rnd());        
            original.setSimGranularity(rnd());
            original.setSimTime(rnd());
            original.setSimpleAttractor(rnd() > rnd()); 
            original.setVthetaSamples(static_cast<int>(rnd()));
            original.setVxSamples(static_cast<int>(rnd()));
            original.setYVels("[-0.3, -0.1, 0.1, 0.3]");

                        
            Location originalLoc(
                ROSBaseLocalPlannerConfiguration::getTemplate(), 
                ROSBaseLocalPlannerConfiguration::getBaseTag());                           
            originalLoc = original;

            
            ROSBaseLocalPlannerConfiguration parsed = extract<ROSBaseLocalPlannerConfiguration>(originalLoc);          
            
            BOOST_REQUIRE_EQUAL(parsed.getGenerator(),          original.getGenerator());
            BOOST_REQUIRE_EQUAL(parsed.getTimestamp(),          original.getTimestamp());
            BOOST_REQUIRE_CLOSE(parsed.getAccLimTheta(),        original.getAccLimTheta(), TOLERANCE);
            
            BOOST_REQUIRE_CLOSE(parsed.getAccLimX(),            original.getAccLimX(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getAccLimY(),            original.getAccLimY(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getAngularSimGranularity(), original.getAngularSimGranularity(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isDwa(),                 original.isDwa());
            
            BOOST_REQUIRE_CLOSE(parsed.getEscapeResetDist(),    original.getEscapeResetDist(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getEscapeResetTheta(),   original.getEscapeResetTheta(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getEscapeVel(),          original.getEscapeVel(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getGdistScale(),         original.getGdistScale(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getHeadingLookahead(),   original.getHeadingLookahead(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isHeadingScoring(),      original.isHeadingScoring());
            BOOST_REQUIRE_CLOSE(parsed.getHeadingScoringTimestep(), original.getHeadingScoringTimestep(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isHolonomicRobot(),      original.isHolonomicRobot());
            BOOST_REQUIRE_CLOSE(parsed.getMaxVelTheta(),        original.getMaxVelTheta(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getMaxVelX(),            original.getMaxVelX(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getMinInPlaceVelTheta(), original.getMinInPlaceVelTheta(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getMinVelTheta(),        original.getMinVelTheta(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getMinVelX(),            original.getMinVelX(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getOccdistScale(),       original.getOccdistScale(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getOscillationResetDist(), original.getOscillationResetDist(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getPdistScale(),         original.getPdistScale(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isRestoreDefaults(),     original.isRestoreDefaults());
            BOOST_REQUIRE_CLOSE(parsed.getSimGranularity(),     original.getSimGranularity(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getSimTime(),            original.getSimTime(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isSimpleAttractor(),     original.isSimpleAttractor());
            BOOST_REQUIRE_EQUAL(parsed.getVthetaSamples(),      original.getVthetaSamples());      
            BOOST_REQUIRE_EQUAL(parsed.getVxSamples(),          original.getVxSamples());
            BOOST_REQUIRE_EQUAL(parsed.getYVels(),              original.getYVels());         
            
        }
        catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_SUITE_END()
