/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/data/person/PerceptList.h"
#include "btl/data/person/Face.h"
#include "btl/data/person/Legs.h"
#include "btl/units/AngleUnit.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(PerceptListTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		// ======================================================================
		// Test with faces.
		// ======================================================================

		PerceptList<Face> original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));

		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.34,RADIAN);
		face->setDistance(10.015,MILLIMETER);
		face->setHeight(-23.567,MILLIMETER);
		original.addPercept(face);

		boost::shared_ptr<Face> face2 = boost::shared_ptr<Face>(new Face);
		face2->setGenerator("facesensor2");
		face2->setAngle(10.34,RADIAN);
		face2->setDistance(110.015,MILLIMETER);
		face2->setHeight(-123.567,MILLIMETER);
		original.addPercept(face2);

		Location originalLoc(PerceptList<Face>::getTemplate(), PerceptList<Face>::getBaseTag());
		originalLoc = original;

		PerceptList<Face> parsed = extract_list<Face>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getPercepts().size(), original.getPercepts().size());
		BOOST_REQUIRE_EQUAL(*(parsed.getPercepts()[0]), *(original.getPercepts()[0]));
		BOOST_REQUIRE_EQUAL(*(parsed.getPercepts()[1]), *(original.getPercepts()[1]));

		// ======================================================================
		// Test with legs.
		// ======================================================================

		PerceptList<Legs> legsOriginal;
		legsOriginal.setGenerator("test");
		legsOriginal.setTimestamp(Timestamp(123, 456));
		
		boost::shared_ptr<Legs> legs = boost::shared_ptr<Legs>(new Legs);
		legs->setAngle(21312.78,DEGREE);
		legs->setDistance(-1.023,METER);
		legs->setAngleVariance(0.45,DEGREE);
		legs->setDistanceVariance(1.34,METER);
		legs->setNumInvolved(2134);
		legs->setInvolved1(4543);
		legs->setInvolved2(1223);
		legs->setPair(true);

		legsOriginal.addPercept(legs);
		
		boost::shared_ptr<Legs> legs2 = boost::shared_ptr<Legs>(new Legs);
		legs2->setAngle(1234.78,DEGREE);
		legs2->setDistance(-3.023,METER);
		legs2->setAngleVariance(1.45,DEGREE);
		legs2->setDistanceVariance(0.34,METER);
		legs2->setNumInvolved(234);
		legs2->setInvolved1(453);
		legs2->setInvolved2(123);
		legs2->setPair(false);

		legsOriginal.addPercept(legs2);
		
		Location legsOriginalLoc(PerceptList<Legs>::getTemplate(), PerceptList<Legs>::getBaseTag());
		legsOriginalLoc = legsOriginal;
		
		PerceptList<Legs> legsParsed = extract_list<Legs>(legsOriginalLoc);

		BOOST_REQUIRE_EQUAL(legsParsed.getGenerator(), legsOriginal.getGenerator());
		BOOST_REQUIRE_EQUAL(legsParsed.getTimestamp(), legsOriginal.getTimestamp());
		BOOST_REQUIRE_EQUAL(legsParsed.getPercepts().size(), legsOriginal.getPercepts().size());
		BOOST_REQUIRE_EQUAL(*(legsParsed.getPercepts()[0]), *(legsOriginal.getPercepts()[0]));
		BOOST_REQUIRE_EQUAL(*(legsParsed.getPercepts()[1]), *(legsOriginal.getPercepts()[1]));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

/*BOOST_AUTO_TEST_CASE(testFileCompatibility) {

 Location fileContents(readFile("PerceptList.xml"), PerceptList::getBaseTag());
 PerceptList parsed = extract<PerceptList>(fileContents);

 BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
 BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
 BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
 BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngle(), tolerance);
 BOOST_REQUIRE_CLOSE((float) 77.23, parsed.getDistance(), tolerance);

 }*/

BOOST_AUTO_TEST_CASE(testGenericUsage) {

	try {

		PerceptList<Face> faces;
		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.34,RADIAN);
		face->setDistance(10.015,MILLIMETER);
		face->setHeight(-23.567,MILLIMETER);
		faces.addPercept(face);

		boost::shared_ptr<Face> face2 = boost::shared_ptr<Face>(new Face);
		face2->setGenerator("facesensor2");
		face2->setAngle(10.34,RADIAN);
		face2->setDistance(110.015,MILLIMETER);
		face2->setHeight(-123.567,MILLIMETER);
		faces.addPercept(face2);

		PerceptList<Percept> list = faces;

		BOOST_REQUIRE_EQUAL(faces.getGenerator(), list.getGenerator());
		BOOST_REQUIRE_EQUAL(faces.getTimestamp(), list.getTimestamp());
		BOOST_REQUIRE_EQUAL(faces.getPercepts().size(), list.getPercepts().size());

		const vector<boost::shared_ptr<Face> > convertedFaces = list.getPerceptsAsType<Face>();
		BOOST_REQUIRE_EQUAL(list.getPercepts().size(), convertedFaces.size());
		BOOST_REQUIRE_EQUAL(*face, *(convertedFaces[0]));
		BOOST_REQUIRE_EQUAL(*face2, *(convertedFaces[1]));

		BOOST_REQUIRE_THROW(list.getPerceptsAsType<Legs>(), BadSharedPtrCast);

		PerceptList<Face> faceList = list;
		BOOST_REQUIRE_THROW(PerceptList<Legs> legList = list, BadSharedPtrCast);

	} catch (std::exception & e) {
		BOOST_FAIL(string(e.what()));
	}

}

BOOST_AUTO_TEST_CASE(testLocationReuse) {

	const std::string attributeName = "testattribute";
	const std::string elementName = "ADDITIONAL";

	PerceptList<Face> original;
	original.setGenerator("test");
	original.setTimestamp(Timestamp(123, 456));

	boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
	face->setGenerator("facesensor");
	face->setAngle(0.34,RADIAN);
	face->setDistance(10.015,MILLIMETER);
	face->setHeight(-23.567,MILLIMETER);
	original.addPercept(face);

	boost::shared_ptr<Face> face2 = boost::shared_ptr<Face>(new Face);
	face2->setGenerator("facesensor2");
	face2->setAngle(10.34,RADIAN);
	face2->setDistance(110.015,MILLIMETER);
	face2->setHeight(-123.567,MILLIMETER);
	original.addPercept(face2);

	Location originalLoc(PerceptList<Face>::getTemplate(), PerceptList<Face>::getBaseTag());
	originalLoc = original;

	originalLoc[attributeName.c_str()] = 23;
	Location additionalElement("<" + elementName + "/>", elementName);
	originalLoc.add(additionalElement);

	// do some parsing and re-serialization into the old location
	PerceptList<Face> parsedFirstTime = extract_list<Face>(originalLoc);
	originalLoc = parsedFirstTime;
	PerceptList<Face> parsedSecondTime = extract_list<Face>(originalLoc);

	// ensure that the parsed data is still the same and additional elements in
	// the xml still exist
	BOOST_REQUIRE_EQUAL(original.size(), parsedSecondTime.size());
	originalLoc[XPath("@" + attributeName)];
	originalLoc[XPath(elementName)];

}

BOOST_AUTO_TEST_SUITE_END()
