package de.unibi.airobots.btl.data.annotation;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.annotation.MapPoint.Kind;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.navigation.NavigationGoalData;
import de.unibi.airobots.btl.data.navigation.NavigationGoalData.GoalType;
import de.unibi.airobots.btl.data.object.ObjectData;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class MapRegionTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		MapRegion original = new MapRegion("abc", "def", "ghi");
		original.setConfidence(6.7);
		original.setHeight(6.8, LengthUnit.METER);
		original.addPoint(1, 2, LengthUnit.METER);
		original.addViewPoint(new NavigationGoalData("sdf", 3, 4, 5,
				GoalType.GLOBAL, LengthUnit.METER, AngleUnit.RADIAN));

		Element doc = new Element(MapRegion.getBaseTag());
		original.fillInto(doc);
		logger.info(doc.toXML());
		System.out.println(doc.toXML());

		MapRegion parsed = new MapRegion();
		MapRegion.fromElement(doc, parsed);

		assertEquals(original.getConfidence(), parsed.getConfidence(), 0.0001);
		assertEquals(original.getHeight(LengthUnit.METER),
				parsed.getHeight(LengthUnit.METER), 0.0001);
		assertEquals(original.getLabel(), parsed.getLabel());
		assertEquals(original.getType(), parsed.getType());
		assertEquals(original.getGroup(), parsed.getGroup());
		assertEquals(original.getPointCount(), parsed.getPointCount());
		assertEquals(original.getPoints().getFirst().getX(LengthUnit.METER),
				parsed.getPoints().getFirst().getX(LengthUnit.METER), 0.0003);
		assertEquals(original.getPoints().getFirst().getY(LengthUnit.METER),
				parsed.getPoints().getFirst().getY(LengthUnit.METER), 0.0003);
		assertEquals(original.getViewPoints().size(), parsed.getViewPoints()
				.size());
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser
				.build(TestUtils.makeTestFileName("MapRegion.xml"));

		MapRegion parsed = new MapRegion();
		MapRegion.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(),
				MapRegion.getBaseTag());

		assertEquals(6.7, parsed.getConfidence(), 0.0001);
		assertEquals(6.8, parsed.getHeight(LengthUnit.METER), 0.0001);
		assertEquals("def", parsed.getLabel());
		assertEquals("abc", parsed.getType());
		assertEquals("ghi", parsed.getGroup());
		assertEquals(1, parsed.getPointCount());
		assertEquals(1, parsed.getPoints().getFirst().getX(LengthUnit.METER),
				0.0003);
		assertEquals(2, parsed.getPoints().getFirst().getY(LengthUnit.METER),
				0.0003);
		assertEquals(1, parsed.getViewPoints().size());

	}

}
