package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.assertEquals;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.geometry.Point3D.Scope;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class BoundingBox3DTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		BoundingBox3D original = new BoundingBox3D();
		Point3D origin = new Point3D(1.0, 1.0, 1.0, LengthUnit.MILLIMETER);
		original.setOrigin(origin);
		Point3D size = new Point3D(2.0, 3.0, 4.0, LengthUnit.MILLIMETER);
		original.setSize(size);
		Rotation3D rotation = new Rotation3D();
		original.setRotation(rotation);

		Document doc = new Document(new Element(BoundingBox3D.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		BoundingBox3D parsed = new BoundingBox3D();
		BoundingBox3D.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getOrigin(), parsed.getOrigin());
		assertEquals(original.getRotation(), parsed.getRotation());
		assertEquals(original.getSize(), parsed.getSize());

		System.out.println(parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {
		
		Point3D origin = new Point3D(1.0, 2.0, 3.0, LengthUnit.MILLIMETER, Scope.LOCAL);
		Point3D size = new Point3D(4.0, 5.0, 6.0, LengthUnit.MILLIMETER, Scope.LOCAL);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("BoundingBox3D.xml"));

		BoundingBox3D parsed = new BoundingBox3D();
		BoundingBox3D.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(),
				BoundingBox3D.getBaseTag());

		assertEquals(new Timestamp(253543797, 253543712), parsed.getTimestamp());

		assertEquals(0.0, parsed.getRotation().getMatrix().m00, 0.00001);
		assertEquals(1.0, parsed.getRotation().getMatrix().m01, 0.00001);
		assertEquals(2.0, parsed.getRotation().getMatrix().m02, 0.00001);
		assertEquals(3.0, parsed.getRotation().getMatrix().m10, 0.00001);
		assertEquals(4.0, parsed.getRotation().getMatrix().m11, 0.00001);
		assertEquals(5.0, parsed.getRotation().getMatrix().m12, 0.00001);
		assertEquals(6.0, parsed.getRotation().getMatrix().m20, 0.00001);
		assertEquals(7.0, parsed.getRotation().getMatrix().m21, 0.00001);
		assertEquals(8.0, parsed.getRotation().getMatrix().m22, 0.00001);
		
		assertEquals(origin, parsed.getOrigin());
		assertEquals(size, parsed.getSize());
	}
}
