package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Element;

import org.junit.Test;

import de.unibi.airobots.btl.data.geometry.Polygon;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class PolygonTest {

	@Test
	public void selfCompatibility() throws Exception {

		Polygon p0 = new Polygon();
		p0.addPoint(0, 0);
		p0.addPoint(2, 0);
		p0.addPoint(0, 2);

		Polygon p1 = new Polygon();
		p1.addPoint(0, 0);
		p1.addPoint(2, 0);

		assertFalse("equals() not working", p0.equals(p1));
		assertFalse("equals() not working", p1.equals(p0));

		p1.addPoint(0, 2);

		assertTrue("equals() not working", p0.equals(p1));
		assertTrue("equals() not working", p1.equals(p0));

		p1.addPoint(0, 3);

		assertFalse("equals() not working", p0.equals(p1));
		assertFalse("equals() not working", p1.equals(p0));
		
		p0.addPoint(1, 2);

		assertFalse("equals() not working", p0.equals(p1));
		assertFalse("equals() not working", p1.equals(p0));

		Element element = new Element(Polygon.getBaseTag());
		p0.fillInto(element);
		
		Polygon p2 = Polygon.fromElement(element);
		
		assertTrue("serializing not working", p0.equals(p2));
	}
}
