package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;

import javax.vecmath.Matrix3d;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.geometry.Rotation3D;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class Rotation3DTest {

	@Test
	public void selfCompatibility() throws Exception {

		double[] m = { 3.0, 45.0, 2.0, 0.5, 0.1, 2.9, 3.89, 0.4, 23.9 };
		Matrix3d mat0 = new Matrix3d(m);
		Rotation3D rot0 = new Rotation3D(mat0);
		Matrix3d mat1 = new Matrix3d(m);
		Rotation3D rot1 = new Rotation3D(mat1);

		assertTrue("equals() not working", rot0.equals(rot1));
		assertTrue("equals() not working", rot1.equals(rot0));

		Element element = new Element(Rotation3D.getBaseTag());
		rot0.fillInto(element);

		Rotation3D rot2 = new Rotation3D();
		Rotation3D.fromElement(element, rot2);

		assertEquals(rot0, rot2);
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("Rotation3D.xml"));

		Rotation3D parsed = new Rotation3D();
		Rotation3D.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(),
				Rotation3D.getBaseTag());

		assertEquals(0.0, parsed.getMatrix().m00, 0.00001);
		assertEquals(1.0, parsed.getMatrix().m01, 0.00001);
		assertEquals(2.0, parsed.getMatrix().m02, 0.00001);
		assertEquals(3.0, parsed.getMatrix().m10, 0.00001);
		assertEquals(4.0, parsed.getMatrix().m11, 0.00001);
		assertEquals(5.0, parsed.getMatrix().m12, 0.00001);
		assertEquals(6.0, parsed.getMatrix().m20, 0.00001);
		assertEquals(7.0, parsed.getMatrix().m21, 0.00001);
		assertEquals(8.0, parsed.getMatrix().m22, 0.00001);
	}
}
