package de.unibi.airobots.btl.data.hardwareinfo;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.hardwareinfo.CameraPositionData;
import de.unibi.airobots.btl.units.*;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class CameraPositionDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		CameraPositionData original = new CameraPositionData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setPan(0.346,AngleUnit.RADIAN);
		original.setTilt(123123.435,AngleUnit.RADIAN);
		original.setZoom(0.33);

		Document doc = new Document(new Element(CameraPositionData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		CameraPositionData parsed = new CameraPositionData();
		CameraPositionData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getPan(AngleUnit.RADIAN), parsed.getPan(AngleUnit.RADIAN), 0.00001);
		assertEquals(original.getTilt(AngleUnit.RADIAN), parsed.getTilt(AngleUnit.RADIAN), 0.00001);
		assertEquals(original.getZoom(), parsed.getZoom(), 0.00001);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("CameraPosition.xml"));

		CameraPositionData parsed = new CameraPositionData();
		CameraPositionData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), CameraPositionData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(0.345, parsed.getPan(AngleUnit.RADIAN), 0.00001);
		assertEquals(123, parsed.getTilt(AngleUnit.RADIAN), 0.00001);
		assertEquals(0.75, parsed.getZoom(), 0.00001);

	}

}
