package de.unibi.airobots.btl.data.map;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.geometry.PrecisePolygon;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import nu.xom.Element;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import org.junit.Test;


/**
 * Test cases for {@link Annotation}.
 *
 * @author lkettenb
 */
public class AnnotationTest {
    @Test
	public void selfCompatibility() throws Exception {
        PrecisePolygon polygon = new PrecisePolygon();
        polygon.addPoint(0.0, 0.0, LengthUnit.METER);
        polygon.addPoint(0.0, 2.0, LengthUnit.METER);
        polygon.addPoint(2.0, 2.0, LengthUnit.METER);
        polygon.addPoint(2.0, 0.0, LengthUnit.METER);
        PositionData coordinates = new PositionData(0.0, 1.0, 2.0, 
                new Timestamp(), LengthUnit.METER, AngleUnit.RADIAN);
        Viewpoint viewpoint = new Viewpoint(coordinates, "table1");
        Viewpoint viewpoint2 = new Viewpoint(coordinates, "table2");
        Annotation annotation = new Annotation("kitchen", polygon, 
                viewpoint, viewpoint2);
        
        Element element = new Element(Annotation.getBaseTag());
        annotation.fillInto(element);
        System.out.println(element.toXML());
        Annotation parsedAnnotation = Annotation.fromElement(element);
        
        assertTrue("Parsing label not working.", 
                annotation.getLabel().equals(parsedAnnotation.getLabel()));
        assertTrue("Parsing polygon not working.", 
                annotation.getPolygon().contains(1.0, 1.0, LengthUnit.METER));
        assertFalse("Parsing polygon not working.", 
                annotation.getPolygon().contains(1.0, 3.0, LengthUnit.METER));
        assertTrue("Parsing viewpoints not working.", 
                annotation.getViewpoints().getFirst().getLabel().equals(
                parsedAnnotation.getViewpoints().getFirst().getLabel()));
        assertTrue("Parsing viewpoints not working.", 
                annotation.getViewpoints().getLast().getLabel().equals(
                parsedAnnotation.getViewpoints().getLast().getLabel()));
    }
}
