package de.unibi.airobots.btl.data.person;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.person.PersonData;
import de.unibi.airobots.btl.units.*;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class PersonDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		PersonData original = new PersonData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setAngle(-0.345,AngleUnit.RADIAN);
		original.setDistance(21.344,LengthUnit.MILLIMETER);
		original.setFacing(true);
		original.setGazing(4.993);
		original.setId(1234);
		original.setName("peter");
		original.setNeighbourDistance(3.45,LengthUnit.MILLIMETER);
		original.setNeighbourId(42);
		original.setPoi(false);
		original.setTalking(true);
		original.setWalking(false);

		Document doc = new Document(new Element(PersonData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		PersonData parsed = new PersonData();
		PersonData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getAngle(AngleUnit.RADIAN), parsed.getAngle(AngleUnit.RADIAN), 0.0001);
		assertEquals(original.getDistance(LengthUnit.MILLIMETER), parsed.getDistance(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(original.isFacing(), parsed.isFacing());
		assertEquals(original.getGazing(), parsed.getGazing(), 0.00001);
		assertEquals(original.getId(), parsed.getId());
		assertEquals(original.getName(), parsed.getName());
		assertEquals(original.getNeighbourDistance(LengthUnit.MILLIMETER), parsed
				.getNeighbourDistance(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getNeighbourId(), parsed.getNeighbourId());
		assertEquals(original.isPoi(), parsed.isPoi());
		assertEquals(original.isTalking(), parsed.isTalking());
		assertEquals(original.isWalking(), parsed.isWalking());

	}
	
	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("Person.xml"));

		PersonData parsed = new PersonData();
		PersonData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), PersonData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(2.1467, parsed.getAngle(AngleUnit.RADIAN), 0.0001);
		assertEquals(1235, parsed.getDistance(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(false, parsed.isFacing());
		assertEquals(Math.PI / 2.0, parsed.getGazing(), 0.00001);
		assertEquals(123, parsed.getId());
		assertEquals("peter", parsed.getName());
		assertEquals(456, parsed.getNeighbourDistance(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(12, parsed.getNeighbourId());
		assertEquals(true, parsed.isPoi());
		assertEquals(true, parsed.isTalking());
		assertEquals(true, parsed.isWalking());

	}

}
