package de.unibi.airobots.btl.data.person;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.person.Shirt;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author ltwardon
 */
public class ShirtTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		try {

			Shirt original = new Shirt();
			original.setGenerator("test");
			final long timestamp = System.currentTimeMillis();
			original.setTimestamp(timestamp);
			original.setAngle(0.45f, AngleUnit.RADIAN);
			original.setDistance(77.23f, LengthUnit.MILLIMETER);
			original.setAngleVariance(0.345f, AngleUnit.RADIAN);
			original.setDistanceVariance(1.34f, LengthUnit.MILLIMETER);

			Document doc = new Document(new Element(Shirt.getBaseTag()));
			original.fillInto(doc.getRootElement());
			logger.info(doc.toXML());

			Shirt parsed = new Shirt();
			Shirt.fromElement(doc.getRootElement(), parsed);

			assertEquals(parsed.getGenerator(), original.getGenerator());
			assertEquals(parsed.getTimestamp(), original.getTimestamp());
			assertEquals(parsed.getAngle(AngleUnit.RADIAN),
					original.getAngle(AngleUnit.RADIAN), 0.0001);
			assertEquals(parsed.getDistance(LengthUnit.MILLIMETER),
					original.getDistance(LengthUnit.MILLIMETER), 0.0001);
			assertEquals(parsed.getAngleVariance(AngleUnit.RADIAN),
					original.getAngleVariance(AngleUnit.RADIAN), 0.0001);
			assertEquals(parsed.getDistanceVariance(LengthUnit.MILLIMETER),
					original.getDistanceVariance(LengthUnit.MILLIMETER), 0.0001);

		} catch (Exception e) {
			fail("Caught exception" + e.getMessage());
		}

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("Shirt.xml"));

		Shirt parsed = new Shirt();
		Shirt.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), Shirt.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(0.45, parsed.getAngle(AngleUnit.RADIAN), 0.0001);
		assertEquals(77.23, parsed.getDistance(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(0.345, parsed.getAngleVariance(AngleUnit.RADIAN), 0.0001);
		assertEquals(1.34, parsed.getDistanceVariance(LengthUnit.MILLIMETER),
				0.0001);

	}

}
