package de.unibi.airobots.btl.data.speechrec;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.ParsingException;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.MicroTimestamp;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.speechrec.SplocAngleData;

/**
 * Test cases for {@link Type}.
 * 
 * @author dschulze <dschulze@techfak.uni-bielefeld.de>
 * @author dklotz
 */
public class SplocAngleDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws ParsingException {

		List<Double> angles = new ArrayList<Double>();
		angles.add(0.1);
		angles.add(0.2);
		angles.add(0.3);			

		SplocAngleData original = new SplocAngleData();
		original.setGenerator("test");
		original.setTimestamp(new Timestamp(123, 456));
		long preciseTime = System.currentTimeMillis();
		original.setReadTime(new MicroTimestamp(preciseTime, TimeUnit.MILLISECONDS));
		original.setAngles(angles);

		Document doc = new Document(new Element(SplocAngleData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		
		logger.info(doc.toXML());
		
		SplocAngleData parsed = new SplocAngleData();
		SplocAngleData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getReadTime(), parsed.getReadTime());
		assertEquals(original.getAngles().size(), parsed.getAngles().size());
		assertEquals(original.getAngles().get(0), parsed.getAngles().get(0), 0.0001);
		assertEquals(original.getAngles().get(1), parsed.getAngles().get(1), 0.0001);
		assertEquals(original.getAngles().get(2), parsed.getAngles().get(2), 0.0001);
		
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("SplocAngleData.xml"));

		SplocAngleData parsed = new SplocAngleData();
		SplocAngleData.fromElement(doc.getRootElement(), parsed);

		List<Double> angles = new ArrayList<Double>();
		angles.add(0.1);
		angles.add(0.2);
		angles.add(0.3);

		Timestamp stamp = new Timestamp(1234, 4567);

		assertEquals("test", parsed.getGenerator());
		assertEquals(stamp, parsed.getTimestamp());
		assertEquals(new MicroTimestamp(123213412, 335435), parsed
				.getReadTime());
		assertEquals(3, angles.size());
		assertEquals(angles.get(0), parsed.getAngles().get(0), 0.0001);
		assertEquals(angles.get(1), parsed.getAngles().get(1), 0.0001);
		assertEquals(angles.get(2), parsed.getAngles().get(2), 0.0001);



	}
	
	@Test
	public void genericFromDocument() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("SplocAngleData.xml"));

		SplocAngleData parsed = Type.fromDocument(doc, SplocAngleData.class);

		List<Double> angles = new ArrayList<Double>();
		angles.add(0.1);
		angles.add(0.2);
		angles.add(0.3);

		Timestamp stamp = new Timestamp(1234, 4567);

		assertEquals("test", parsed.getGenerator());
		assertEquals(stamp, parsed.getTimestamp());
		assertEquals(new MicroTimestamp(123213412, 335435), parsed
				.getReadTime());
		assertEquals(3, angles.size());
		assertEquals(angles.get(0), parsed.getAngles().get(0), 0.0001);
		assertEquals(angles.get(1), parsed.getAngles().get(1), 0.0001);
		assertEquals(angles.get(2), parsed.getAngles().get(2), 0.0001);



	}

}
