package de.unibi.airobots.btl.data.speechrec;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.speechrec.Voice;
import de.unibi.airobots.btl.units.*;

/**
 * Test cases for {@link Type}.
 * 
 * @author ltwardon
 */
public class VoiceTest {
	
	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		
		try{
			
			Voice original = new Voice();
			original.setGenerator("test");
			final long timestamp = System.currentTimeMillis();
			original.setTimestamp(timestamp);
			original.setAngle(21312.78f,AngleUnit.RADIAN);
			original.setRuntimeDifference(1023,TimeUnit.MICROSECONDS);
			original.setAngleVariance(0.45f, AngleUnit.RADIAN);
			original.setRuntimeDifferenceVariance(1340, TimeUnit.MICROSECONDS);
	
			Document doc = new Document(
					new Element(Voice.getBaseTag()));
			original.fillInto(doc.getRootElement());
			logger.info(doc.toXML());
			
			Voice parsed = new Voice();
			Voice.fromElement(doc.getRootElement(), parsed);
			
			assertEquals(parsed.getGenerator(), original.getGenerator());
			assertEquals(parsed.getTimestamp(), original.getTimestamp());
			assertEquals(parsed.getAngle(AngleUnit.RADIAN), original.getAngle(AngleUnit.RADIAN), 0.0001);
			assertEquals(parsed.getRuntimeDifference(TimeUnit.MILLISECONDS), original.getRuntimeDifference(TimeUnit.MILLISECONDS), 0.0001);
			assertEquals(parsed.getAngleVariance(AngleUnit.RADIAN), original.getAngleVariance(AngleUnit.RADIAN), 0.0001);
			assertEquals(parsed.getRuntimeDifferenceVariance(TimeUnit.MICROSECONDS), original.getRuntimeDifferenceVariance(TimeUnit.MICROSECONDS));
			
			assertEquals(parsed, original);
			parsed.setGenerator("trash");
			assertFalse(parsed.equals(original));
			parsed.setGenerator(original.getGenerator());
			parsed.setRuntimeDifference(999900,TimeUnit.MICROSECONDS);
			assertFalse(parsed.equals(original));
			
		}catch(Exception e){
			fail("Caught exception" + e.getMessage());
		}
	}
	
	@Test
	public void fileCompatibility() throws Exception {
		
		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("Voice.xml"));

		Voice parsed = new Voice();
		Voice.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), Voice
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(0.45, parsed.getAngle(AngleUnit.RADIAN), 0.0001);
		assertEquals(77230, parsed.getRuntimeDifference(TimeUnit.MICROSECONDS));
		assertEquals(0.345, parsed.getAngleVariance(AngleUnit.RADIAN), 0.0001);
		assertEquals(1340, parsed.getRuntimeDifferenceVariance(TimeUnit.MICROSECONDS));
		
	}
		
}
