package de.unibi.airobots.btl.data.vision3d;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;

import javax.vecmath.Matrix3d;

import org.junit.Test;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.data.geometry.Rotation3D;
import de.unibi.airobots.btl.data.vision3d.PointCloud;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link PointCloud}.
 * 
 * @author plueckin
 */
public class PointCloudTest {

	@Test
	public void selfCompatibility() throws Exception {

		PointCloud pc = new PointCloud();

		Point3D p0 = new Point3D(1, 2, 3, LengthUnit.MILLIMETER);
		Point3D p1 = new Point3D(1, 2, 3, LengthUnit.MILLIMETER);
		Point3D p2 = new Point3D(0.1, 0.2, 0.3, LengthUnit.MILLIMETER);
		pc.addPoint(p0);
		pc.addPoint(p1);
		pc.addPoint(p2);

		assertTrue("equals() not working",
				pc.getPoints().get(0).equals(pc.getPoints().get(1)));
		assertTrue("equals() not working",
				pc.getPoints().get(1).equals(pc.getPoints().get(0)));
		assertFalse("equals() not working",
				pc.getPoints().get(0).equals(pc.getPoints().get(2)));

		pc.setSensorOrigin(new Point3D(0, 0, 0, LengthUnit.MILLIMETER));

		Matrix3d rot = new Matrix3d(1, 2, 3, 4, 4, 5, 6, 7, 8);
		pc.setRotation(new Rotation3D(rot));

		Element element = new Element(PointCloud.getBaseTag());
		pc.fillInto(element);

		PointCloud parsed = new PointCloud();
		PointCloud.fromElement(element, parsed);

		assertTrue("serializing not working", pc.equals(parsed));
	}

	@Test
	public void fileCompatibility() throws Exception {

		PointCloud pc = new PointCloud();

		Point3D p0 = new Point3D(12, 12, 81, LengthUnit.MILLIMETER);
		Point3D p1 = new Point3D(1.6, 51, 511, LengthUnit.MILLIMETER);
		Point3D p2 = new Point3D(14, 34, 66, LengthUnit.MILLIMETER);
		Point3D p3 = new Point3D(82, 16, 44, LengthUnit.MILLIMETER);

		pc.addPoint(p0);
		pc.addPoint(p1);
		pc.addPoint(p2);
		pc.addPoint(p3);

		pc.setSensorOrigin(new Point3D(41.3, 12.1, 21.84, LengthUnit.MILLIMETER));

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("PointCloud.xml"));

		PointCloud parsed = new PointCloud();
		PointCloud.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(),
				PointCloud.getBaseTag());

		assertEquals(new Timestamp(127194l, 127199l), parsed.getTimestamp());


		assertEquals(pc.getSensorOrigin().getX(LengthUnit.MILLIMETER), parsed
				.getSensorOrigin().getX(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(pc.getSensorOrigin().getY(LengthUnit.MILLIMETER), parsed
				.getSensorOrigin().getY(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(pc.getSensorOrigin().getZ(LengthUnit.MILLIMETER), parsed
				.getSensorOrigin().getZ(LengthUnit.MILLIMETER), 0.0001);


		assertEquals(pc.getPoints().get(0).getX(LengthUnit.MILLIMETER), parsed
				.getPoints().get(0).getX(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(pc.getPoints().get(1).getZ(LengthUnit.MILLIMETER), parsed
				.getPoints().get(1).getZ(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(pc.getPoints().get(2).getY(LengthUnit.MILLIMETER), parsed
				.getPoints().get(2).getY(LengthUnit.MILLIMETER), 0.0001);

	}
}
